<?php
// webhook.php
//
// Use this sample code to handle webhook events in your integration.
//
// 1) Paste this code into a new file (webhook.php)
//
// 2) Install dependencies
//   composer require stripe/stripe-php
//
// 3) Run the server on http://localhost:4242
//   php -S localhost:4242
require_once dirname(dirname(__FILE__),2) . '/stripe/stripe/init.php';
require_once dirname(dirname(__FILE__),2) . '/classes/dateTimeService.php';
require_once dirname(dirname(__FILE__),2) . '/POS/classes/pos.php';
require_once dirname(dirname(__FILE__),2) . '/POS/classes/posCart.php';
require_once dirname(dirname(__FILE__),2) . '/POS/classes/posItemsPurchased.php';
require_once dirname(dirname(__FILE__)) . '/terminal/classes/terminal.php';
require_once dirname(dirname(__FILE__)) . '/terminal/classes/termPdf.php';
require_once dirname(dirname(__FILE__)) . '/terminal/classes/termEmail.php';
require_once dirname(dirname(__FILE__),2) . '/POS/classes/posPdf.php';
require_once dirname(dirname(__FILE__),2) . '/POS/classes/posEmail.php';
require_once dirname(dirname(__FILE__), 2) . '/variables/variablesClass.php';
require_once dirname(dirname(__FILE__)) . '/LobbyPOS/classes/posDbServices.php';
// See your keys here: https://dashboard.stripe.com/apikeys

$terminal = new terminal();


\Stripe\Stripe::setApiKey($terminal->getApiKey());
// Replace this endpoint secret with your endpoint's unique secret
// If you are testing with the CLI, find the secret by running 'stripe listen'
// If you are using an endpoint defined with the API or dashboard, look in your webhook settings
// at https://dashboard.stripe.com/webhooks
$endpoint_secret = $terminal->getEndpointSecret();
$payload = @file_get_contents('php://input');
$event = null;

if($payload) {
    try {
        $event = \Stripe\Event::constructFrom(
            json_decode($payload, true)
        );
        //\Stripe\Webhook::constructEvent($payload, $signature, $endpoint_secret);
    } catch (\UnexpectedValueException $e) {
        // Invalid payload
        error_log('⚠️  Webhook error while parsing basic request.');
        http_response_code(400);
        exit();
    }
    // Only verify the event if there is an endpoint secret defined
    // Otherwise use the basic decoded event
    $sig_header = $_SERVER['HTTP_STRIPE_SIGNATURE'];
    try {
        $event = \Stripe\Webhook::constructEvent(
            $payload, $sig_header, $endpoint_secret
        );
    } catch (\Stripe\Exception\SignatureVerificationException $e) {
        // Invalid signature
        error_log('⚠️  Webhook error while validating signature.');
        http_response_code(400);
        exit();
    }
}
// Handle the event
if($event){

    switch ($event->type) {

        case 'terminal.reader.action_succeeded':
            $terminalReader = $event->data->object;
            // Assume you have a way to get the PaymentIntent ID associated with this terminal action
            // This might be stored in your database or received from another source
            $paymentIntentId = $terminalReader->action->process_payment_intent->payment_intent; // Replace with actual logic to retrieve the ID

            // Retrieve the PaymentIntent details from Stripe
            try {
                $paymentIntent = \Stripe\PaymentIntent::retrieve($paymentIntentId);
                // Calculate the tip amount
                $originalAmount = $paymentIntent->amount; // The original amount before tip
                $amountReceived = $paymentIntent->amount_received; // The total amount including the tip
                $tipAmount = $amountReceived - $originalAmount;

                // Optionally, save the tip amount in the metadata
                // Update the PaymentIntent with the tip amount in metadata

                $description = $paymentIntent->description;
                $metadata = $paymentIntent->metadata;
                $amount_details = $paymentIntent->amount_details;
                $amount = $paymentIntent->amount;
                $currency = $paymentIntent->currency;
                $userID = $metadata['userId'];
                $programLocation = $metadata['programLocation'];
                $tipOb = $amount_details['tip'];
                $tip = $tipOb['amount'];
                $updatedPaymentIntent = \Stripe\PaymentIntent::update(
                    $paymentIntentId,
                    [
                        'metadata' => [
                            'tipped_amount' => $tip,
                        ],
                    ]
                );
                if($programLocation == "posTerminal"){
                    storePOSResults($terminalReader->action->status, $terminalReader->action->status, $paymentIntent->id,$terminalReader->id, $userID);
                }else {
                    storeResults($terminalReader->action->status, $terminalReader->action->status, $paymentIntent->id,$terminalReader->id, $userID);
                }

                $status = $terminal->processPayment($paymentIntent->id);
                if($status == 'Payment successful'){
                    //find charge data
                    $brand = '';
                    $last4 = '';
                    try {
                        // Replace 'ch_...' with the actual Charge ID
                        $charge = \Stripe\Charge::retrieve($paymentIntent->latest_charge);

                        // Access card details
                        $paymentMethodDetails = $charge->payment_method_details;
                        $cardDetails = $paymentMethodDetails->card_present;
                        $brand = $cardDetails->brand;
                        $last4 = $cardDetails->last4;

                    } catch (\Stripe\Exception\ApiErrorException $e) {
                        // Handle API error
                        if($programLocation == "posTerminal"){
                            storePOSResults("Error retrieving Charge: ",$e->getMessage(), $terminalReader->action->process_payment_intent->payment_intent,$terminalReader->id, $userID);
                        }else {
                            storeResults("Error retrieving Charge: ",$e->getMessage(), $terminalReader->action->process_payment_intent->payment_intent,$terminalReader->id, $userID);
                        }

                    }

                    //check if POS
                    if($programLocation == "posTerminal"){
                        //make posReciept
                        //insert into posdb
                        $pos = new pos();
                        $pos->setPosPurchasesDescription($description);
                        $pos->setPosPurchasesPrice($amount);
                        $pos->setPosPurchasesCardType($brand);
                        $pos->setPosPurchasesLast4($last4);
                        $pos->setPosPurchasesUserId($userID);
                        $pos->setPosPurchasesPosId($terminalReader->id);
                        $pos->setPosPurchasesPosName($terminalReader->label);
                        $pos->setPosPurchasesTID($paymentIntent->id);
                        $pos->setPosPurchaseTip($tip);
                        $dt = new dateTimeService();
                        $timeStamp = $dt->createCurrentTimeStamp();
                        $pos->setPosPurchasesTimeStamp($timeStamp);
                        $storeTransResults = $pos->storeCompletedTransaction();
                        $posItemsPurchased = new posItemsPurchased();
                        $posItemsPurchased->addItemsFromCart($userID, $pos->getPosPurchasesId());
                        $cart = new posCart();
                        $cart->removeItemFromCartByUser($userID);

                        if (strpos($storeTransResults, "error") == false) {
                            if ($programLocation == "posTerminal") {
                                //create reciept
                                //$posPdf = new posPdf();
                                //$fileName = $posPdf->createPDFFIle($pos);
                                //update filename
                                //$pos->setPosPurchasesFileName($fileName);
                                $pos->updatePurchase($paymentIntentId);

                                //$posEmail = new posEmail();
                                //$posEmail->sendEmail($pos);
                            }

                        } else {
                            if($programLocation == "posTerminal"){
                                storePOSResults("Error Adding Transaction: ", $storeTransResults, $paymentIntent->id, $terminalReader->id, $userID);
                            }else {
                                storeResults("Error Adding Transaction: ", $storeTransResults, $paymentIntent->id, $terminalReader->id, $userID);
                            }

                        }

                    } //check if POS
                    elseif ($programLocation == "lobbyTerminal"){
                        //make posReciept
                        //insert into posdb
                        require_once dirname(dirname(__FILE__),2) . '/LobbyPOS/classes/pos.php';

                        $pos = new pos();
                        $pos->setPosPurchasesDescription($description);
                        $pos->setPosPurchasesPrice($amount);
                        $pos->setPosPurchasesCardType($brand);
                        $pos->setPosPurchasesLast4($last4);
                        $pos->setPosPurchasesUserId($userID);
                        $pos->setPosPurchasesPosId($terminalReader->id);
                        $pos->setPosPurchasesPosName($terminalReader->label);
                        $pos->setPosPurchasesTID($paymentIntent->id);
                        $pos->setPosPurchaseTip($tip);
                        $dt = new dateTimeService();
                        $timeStamp = $dt->createCurrentTimeStamp();
                        $pos->setPosPurchasesTimeStamp($timeStamp);
                        $storeTransResults = $pos->storeCompletedTransaction();
                        $posItemsPurchased = new posItemsPurchased();
                        $posItemsPurchased->addItemsFromCart($userID, $pos->getPosPurchasesId());
                        $cart = new posCart();
                        $cart->removeItemFromCartByUser($userID);

                        if (strpos($storeTransResults, "error") == false) {
                            if ($programLocation == "lobbyTerminal") {
                                //create reciept
                                //$posPdf = new posPdf();
                                //$fileName = $posPdf->createPDFFIle($pos);
                                //update filename
                                //$pos->setPosPurchasesFileName($fileName);
                                $pos->updatePurchase($paymentIntentId);

                                //$posEmail = new posEmail();
                                //$posEmail->sendEmail($pos);
                            }

                        } else {
                            if($programLocation == "lobbyTerminal"){
                                storePOSResults("Error Adding Transaction: ", $storeTransResults, $paymentIntent->id, $terminalReader->id, $userID);
                            }else {
                                storeResults("Error Adding Transaction: ", $storeTransResults, $paymentIntent->id, $terminalReader->id, $userID);
                            }

                        }

                    }
                    else {
                        //make Reciept
                        //insert into db
                        $terminal->setTerminalPurchasesDescription($description);
                        $terminal->setTerminalPurchasesPrice($amount);
                        $terminal->setTerminalPurchasesCardType($brand);
                        $terminal->setTerminalPurchasesLast4($last4);
                        $terminal->setTerminalPurchasesUserId($userID);
                        $terminal->setTerminalPurchasesTerminalId($terminalReader->id);
                        $terminal->setTerminalPurchasesTerminalName($terminalReader->label);
                        $terminal->setTerminalPurchasesTID($paymentIntent->id);
                        $dt = new dateTimeService();
                        $timeStamp = $dt->createCurrentTimeStamp();
                        $terminal->setTerminalPurchasesTimeStamp($timeStamp);

                        $storeTransResults = $terminal->storeCompletedTransaction();
                        if (strpos($storeTransResults, "error") == false) {
                            if ($programLocation == "CCTerminal") {
                                //create reciept
                                $termPdf = new termPdf();
                                $fileName = $termPdf->createPDFFIle($terminal);
                                //update filename
                                $terminal->setTerminalPurchasesFileName($fileName);
                                $terminal->updatePurchase($paymentIntentId);

                                $termEmail = new termEmail();
                                $termEmail->sendEmail($terminal);
                            }

                        } else {
                            if($programLocation == "posTerminal"){
                                storePOSResults("Error Adding Transaction: ", $storeTransResults, $paymentIntent->id, $terminalReader->id, $userID);
                            }else {
                                storeResults("Error Adding Transaction: ", $storeTransResults, $paymentIntent->id, $terminalReader->id, $userID);
                            }

                        }
                    }//close if not posTerminal
                }else{
                    //log error
                    if($programLocation == "posTerminal"){
                        storePOSResults("Error Processing Payment: ",$status,  $paymentIntent->id, $terminalReader->id, $userID);
                    }else {
                        storeResults("Error Processing Payment: ",$status,  $paymentIntent->id, $terminalReader->id, $userID);
                    }

                }
            } catch (\Stripe\Exception\ApiErrorException $e) {
                // Handle API error
                http_response_code(400);
                storeResults("PayIntent Error: ",$e->getMessage(), $terminalReader->action->process_payment_intent->payment_intent,$terminalReader->id, $userID);
                storePOSResults("PayIntent Error: ",$e->getMessage(), $terminalReader->action->process_payment_intent->payment_intent,$terminalReader->id, $userID);
            }
            break;
        case 'terminal.reader.action_failed':
            $terminalReader = $event->data->object;
            // Handle terminal reader action success
            storeResults($terminalReader->action->failure_code,$terminalReader->action->failure_message, $terminalReader->action->process_payment_intent->payment_intent,$terminalReader->id, -1);
            storePOSResults($terminalReader->action->failure_code,$terminalReader->action->failure_message, $terminalReader->action->process_payment_intent->payment_intent,$terminalReader->id, -1);
            break;
        default:
            $paymentIntent = $event->data->object;
            // Unexpected event type
            storeResults("Error: ".$event->type," Try another card.", $paymentIntent,$paymentIntent->id, -1);
            storePOSResults("Error: ".$event->type," Try another card.", $paymentIntent,$paymentIntent->id, -1);
            break;

    }
}
http_response_code(200);

function storeResults($code, $message, $pi, $tId, $uid){
    $terminal = new terminal();
    $terminal->setTerminalStatusCode($code);
    $terminal->setTerminalStatusMessage($message);
    $terminal->setTerminalStatusPi($pi);
    $terminal->setTerminalStatusTerminalID($tId);
    $terminal->setTerminalStatusUserID($uid);
    if($terminal->findResults($pi) == ''){
        $status = $terminal->storeResults();
        if(strpos($status, 'error') == false){
            error_log($status);
        }
    }else{
        $status = $terminal->updateResults($pi);
        if(strpos($status, 'error') == false){
            error_log($status);
        }
    }

}

function storePOSResults($code, $message, $pi, $tId, $uid){
    $pos = new pos();
    $pos->setPosStatusCode($code);
    $pos->setPosStatusMessage($message);
    $pos->setPosStatusPi($pi);
    $pos->setPosStatusPosID($tId);
    $pos->setPosStatusUserID($uid);
    if($pos->findResults($pi) == ''){
        $status = $pos->storeResults();
        if(strpos($status, 'error') == false){
            error_log($status);
        }
    }else{
        $status = $pos->updateResults($pi);
        if(strpos($status, 'error') == false){
            error_log($status);
        }
    }

}

?>

