<?php
require_once dirname(dirname(__FILE__),2) . '/variables/variablesClass.php';

$vars = new variables();
set_time_limit(0);

$address = '127.0.0.1';
$port = 8080;

$allowed_origins = [$vars->getWebURL(), 'https://localhost:3000']; // Add allowed origins here

$server = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
socket_set_option($server, SOL_SOCKET, SO_REUSEADDR, 1);
socket_bind($server, $address, $port) or die('Could not bind to address');
socket_listen($server) or die('Could not set up socket listener');

$clients = [];

function handshake($client, $headers) {
    global $allowed_origins;

    // Extract the Origin header
    $lines = explode("\r\n", $headers);
    $origin = '';
    foreach ($lines as $line) {
        if (strpos($line, 'Origin:') === 0) {
            $origin = trim(str_replace('Origin:', '', $line));
            break;
        }
    }

    // Check if the origin is allowed
    if (!in_array($origin, $allowed_origins)) {
        socket_close($client);
        return false;
    }

    // Perform WebSocket handshake
    foreach ($lines as $line) {
        if (strpos($line, 'Sec-WebSocket-Key:') !== false) {
            $key = trim(str_replace('Sec-WebSocket-Key:', '', $line));
        }
    }

    $acceptKey = base64_encode(pack('H*', sha1($key . '258EAFA5-E914-47DA-95CA-C5AB0DC85B11')));
    $upgrade = "HTTP/1.1 101 Switching Protocols\r\n"
        . "Upgrade: websocket\r\n"
        . "Connection: Upgrade\r\n"
        . "Sec-WebSocket-Accept: $acceptKey\r\n\r\n";
    socket_write($client, $upgrade, strlen($upgrade));
    return true;
}

function sendMessage($client, $msg) {
    $frame = chr(129) . chr(strlen($msg)) . $msg;
    socket_write($client, $frame, strlen($frame));
}

while (true) {
    $read = [$server];
    $read = array_merge($read, $clients);
    $write = $except = null;

    socket_select($read, $write, $except, null);

    if (in_array($server, $read)) {
        $client = socket_accept($server);
        $header = socket_read($client, 1024);
        if (handshake($client, $header)) {
            $clients[] = $client;
        }
        $found_socket = array_search($server, $read);
        unset($read[$found_socket]);
    }

    foreach ($read as $client) {
        $input = socket_read($client, 1024, PHP_BINARY_READ);
        if ($input === false) {
            unset($clients[array_search($client, $clients)]);
            socket_close($client);
            continue;
        }
        sendMessage($client, "Hello, WebSocket!");
    }
}

socket_close($server);
?>
